/*
 * File         : BV_Member.cpp
 * Author       : Tomasz Stachowiak
 * Created      : Thu May 21 19:44:41 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include <bstring.h>
#include <semaphore.h>

#include <BVN/Reactor.h>
#include <BVN/Connector.h>
#include <BVN/SOCK_Connector.h>
#include <BVN/SOCK_Library.h>

#include "rtp.h"
#include "H263.h"
#include "DisplayWindowMainWindow.h"
#include "BV_Svc_Handler.h"
#include "UserData.h"

#include "BV_Member.h"

#define BV_PORT 5000

BV_Member::BV_Member() : stream_(0)
{
  dpy_win = NULL;
  display_on_ = 0;
  audio_data_ = NULL;
  video_data_ = NULL;
  open_dpy_ = 0;
  init_dpy_ = 0;
  close_dpy_ = 0;
  display_ = 0;
  sem_init(&sem_, 0, 1);

  //open_display();
}

BV_Member::BV_Member(SOCK_Stream *stream, int id, char *name, int host) 
  : stream_(stream), id_(id), host_(host)
{
  strcpy(name_, name);
  dpy_win = NULL;
  display_on_ = 0;
  audio_data_ = NULL;
  video_data_ = NULL;
  open_dpy_ = 0;
  init_dpy_ = 0;
  close_dpy_ = 0;
  display_ = 0;
  sem_init(&sem_, 0, 1);

  //open_display();
}

BV_Member::BV_Member(char *name, int host) 
  : host_(host)
{
  strcpy(name_, name);
  stream_ = 0;
  dpy_win = NULL;
  display_on_ = 0;
  audio_data_ = NULL;
  video_data_ = NULL;
  open_dpy_ = 0;
  init_dpy_ = 0;
  close_dpy_ = 0;
  display_ = 0;
  sem_init(&sem_, 0, 1);

  //open_display();
}

BV_Member::BV_Member(BV_Member *mbr)
{
  id_ = mbr->get_id();
  host_ = mbr->get_host();
  mbr->get_name(name_);
  stream_ = mbr->get_stream();
  display_on_ = mbr->display_on_;
  internal_id_ = mbr->get_internal_id();
  open_dpy_ = 0;
  init_dpy_ = 0;
  close_dpy_ = 0;
  display_ = 0;
  sem_init(&sem_, 0, 1);

  //open_display();
}

BV_Member::~BV_Member()
{
  close_display();
}

void BV_Member::operator= (BV_Member &mbr)
{
  id_ = mbr.get_id();
  host_ = mbr.get_host();
  mbr.get_name(name_);
  stream_ = mbr.get_stream();
  display_on_ = mbr.display_on_;
  internal_id_ = mbr.get_internal_id();
  open_dpy_ = 0;
  init_dpy_ = 0;
  close_dpy_ = 0;
  display_ = 0;
  sem_init(&sem_, 0, 1);

  open_display();
}

int BV_Member::connect(Reactor *reactor)
{
  struct sockaddr_in addr;

  BV_Svc_Handler *sh = new BV_Svc_Handler();
  Connector<BV_Svc_Handler, SOCK_Connector> *conn 
    = new Connector<BV_Svc_Handler, SOCK_Connector>(reactor);

  bzero(&addr, sizeof(addr));
  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = htonl(host_);
  addr.sin_port = htons(BV_PORT);
  if(conn->connect(&addr, sh) == -1)
    {
      stream_ = 0;
      return -1;
    }

  stream_ = sh->peer();
  return 0;
}

void BV_Member::close()
{
  if(stream_ != 0)
    stream_->close();
}

int BV_Member::get_id()
{
  return id_;
}

void BV_Member::set_id(int id)
{
  id_ = id;
}

void BV_Member::get_name(char *name)
{
  strcpy(name, name_);
}

void BV_Member::set_name(char *name)
{
  strcpy(name_, name);
}

int BV_Member::get_host()
{
  return host_;
}

int BV_Member::get_string(char *str)
{
  char tmp[50];

  strcpy(str, name_);
  SOCK_Library::gethostname(host_, tmp);
  strcat(str, "@");
  strcat(str, tmp);

  return 0;  
}

int BV_Member::send_msg(BV_Message *msg)
{
  return msg->send(stream_);
}

int BV_Member::get_participant(Participant *part)
{
  part->handle = id_;
  part->host = host_;
  strcpy(part->user, name_);

  return 0;
}

SOCK_Stream *BV_Member::get_stream()
{
  return stream_;
}

void BV_Member::set_stream(SOCK_Stream *stream)
{
  stream_ = stream;
}

void BV_Member::set_internal_id(int internal_id)
{
  internal_id_ = internal_id;
}

int BV_Member::get_internal_id()
{
  return internal_id_;
}

void BV_Member::update_audio_stat(RTPheader *header, int size)
{
  if(audio_data_ == NULL)
    audio_data_ = new UserData(header, size);
  else
    audio_data_->frame(header, size);
}

void BV_Member::update_video_stat(RTPheader *header, int size)
{
  static int interval = 2;

  sem_wait(&sem_);
  if(video_data_ == NULL)
    video_data_ = new UserData(header, size);
  else
    video_data_->frame(header, size);
  sem_post(&sem_);

  /*
  if(dpy_win != NULL)
    {
      if(interval == 0)
	{
	  char text[100], text1[100];
	  sprintf(text, "%4.0f kbps", video_data_->getKbps());
	  sprintf(text1, "%2.1f fps", video_data_->getFps());
	  dpy_win->setStatistics(text, text1);
	}

      interval = (interval + 1) % 3;
    }
  */
}

UserData *BV_Member::get_audio_data()
{
  return audio_data_;
}

UserData *BV_Member::get_video_data()
{
  return video_data_;
}

int BV_Member::open_display()
{
  char str[100];
  static int dpy_ = 0;

    

  if(dpy_win == NULL)
    {
      this->get_string(str);
      dpy_win = new DisplayWindowMainWindow(str);
      //dpy_win->show();
      init_dpy_ = 1;
    }
  else
    return -1;

  return 0;
}

void BV_Member::close_display()
{
  if(dpy_win != NULL)
    {
      delete dpy_win;
      dpy_win = NULL;    
    }
}

void BV_Member::show_display()
{
  if(dpy_win != NULL)
    {
      dpy_win->show();
    }
}

void BV_Member::hide_display()
{
  if(dpy_win != NULL)
    {
      dpy_win->hide();
    }
}

int BV_Member::display(char *frame)
{
  static int interval = 2;

  if((dpy_win != NULL) && (init_dpy_ == 0))
    {
      displayYUV411(internal_id_, XtWindow(dpy_win->viewWidget()), (char *)frame);
      display_ = 0;

      
      if(interval == 0)
	{
	  if(video_data_ != NULL)
	    {
	      char text[100], text1[100];
	      sprintf(text, "%4.0f kbps", video_data_->getKbps());
	      sprintf(text1, "%2.1f fps", video_data_->getFps());
	      dpy_win->setStatistics(text, text1);
	    }
	  else
	      dpy_win->setStatistics("0 kbps", "0 fps");

	}

      interval = (interval + 1) % 3;
      
    }

  if(init_dpy_ == 1)
    {
      init_display2(XtWindow(dpy_win->viewWidget()));
      init_dpy_ = 0;
    }
  if(dpy_win == NULL)
    open_display();

  //  sem_post(&sem_);
  return 0;
}

void BV_Member::open_dpy_req()
{
  sem_wait(&sem_);
  open_dpy_ = 1;
  sem_post(&sem_);
}

void BV_Member::close_dpy_req()
{
  sem_wait(&sem_);
  close_dpy_ = 1;
  sem_post(&sem_);
}

void BV_Member::display_req(char *frame)
{
  sem_wait(&sem_);

  memcpy(frame_, frame, 176*144*3/2);
  display_ = 1;

  sem_post(&sem_);
}

int BV_Member::handle_events()
{
  struct timeval time;

  time.tv_sec = 0;
  time.tv_usec = 500000;

  sem_wait(&sem_);

  if(open_dpy_ == 1)
  {
    show_display();
    open_dpy_ = 0;
  }
  else
    if(display_ == 1)
      {
	display(frame_);
	display_ = 0;
      }
    else
      if(close_dpy_ == 1)
	{
	  hide_display();
	  close_dpy_ = 0;
	}

  sem_post(&sem_);
}
